/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.server.service.impl;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.appspy.core.data.CollectedData;
import org.appspy.server.service.DataCollectionService;
import org.springframework.transaction.PlatformTransactionManager;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class DataCollectionServiceImpl implements DataCollectionService {
	
	protected PlatformTransactionManager mTransactionManager = null;
	protected DataProcessor mDataProcessor = null;
	
	protected static final Log sLog = LogFactory.getLog(DataCollectionServiceImpl.class);

	/* (non-Javadoc)
	 * @see org.appspy.core.server.service.DataCollectionService#collectData(java.util.List)
	 */
	public void collectData(List<CollectedData> collectedDatas) {

		sLog.info("Start of data collection : " + collectedDatas);
		
		for (CollectedData collectedData : collectedDatas) {
			try
			{
				mDataProcessor.process(collectedData, null);
			}
			catch (Throwable t)
			{
				sLog.error("Error during data collection", t);
			}
		}
		
		sLog.info("End of data collection : " + collectedDatas);
	}


	/**
	 * @return the dataProcessor
	 */
	public DataProcessor getDataProcessor() {
		return mDataProcessor;
	}


	/**
	 * @param dataProcessor the dataProcessor to set
	 */
	public void setDataProcessor(DataProcessor dataProcessor) {
		mDataProcessor = dataProcessor;
	}


	/**
	 * @return the transactionManager
	 */
	public PlatformTransactionManager getTransactionManager() {
		return mTransactionManager;
	}

	/**
	 * @param transactionManager the transactionManager to set
	 */
	public void setTransactionManager(PlatformTransactionManager transactionManager) {
		mTransactionManager = transactionManager;
	}

}
