/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.server.service.impl;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Element;
import net.sf.ehcache.constructs.blocking.CacheEntryFactory;
import net.sf.ehcache.constructs.blocking.SelfPopulatingCache;

import org.appspy.server.bo.Application;
import org.appspy.server.bo.ApplicationVersion;
import org.appspy.server.bo.ClientHost;
import org.appspy.server.bo.DelayType;
import org.appspy.server.bo.Environment;
import org.appspy.server.bo.HitResult;
import org.appspy.server.bo.Host;
import org.appspy.server.bo.Session;
import org.appspy.server.bo.URL;
import org.appspy.server.bo.User;
import org.appspy.server.bo.Webapp;
import org.appspy.server.dao.ApplicationDao;
import org.appspy.server.dao.EnvironmentDao;
import org.appspy.server.dao.PerfDao;
import org.appspy.server.service.impl.cache.AppVersionKey;
import org.appspy.server.service.impl.cache.EnvHostKey;
import org.appspy.server.service.impl.cache.OrgEnvKey;
import org.appspy.server.service.impl.cache.SessionKey;
import org.appspy.server.service.impl.cache.URLKey;
import org.appspy.server.service.impl.cache.WebappKey;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.transaction.PlatformTransactionManager;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class DataCollectionUtilsImpl implements DataCollectionUtils, InitializingBean {

	protected ApplicationDao mApplicationDao = null;
	protected EnvironmentDao mEnvironmentDao = null;
	protected PerfDao mPerfDao = null;
	
	protected CacheManager mCacheManager = null;
	protected SelfPopulatingCache mVersionCache = null;
	protected SelfPopulatingCache mEnvironmentCache = null;
	protected SelfPopulatingCache mHostCache = null;
	protected SelfPopulatingCache mWebappCache = null;
	protected SelfPopulatingCache mClientHostCache = null;
	protected SelfPopulatingCache mUserCache = null;
	protected SelfPopulatingCache mSessionCache = null;
	protected SelfPopulatingCache mURLCache = null;
	protected SelfPopulatingCache mHitResultCache = null;
	protected SelfPopulatingCache mDelayTypeCache = null;
	
	protected DataProcessor mDataProcessor = null;
	protected PlatformTransactionManager mTransactionManager = null;
	
	/**
	 * @return the dataProcessor
	 */
	public DataProcessor getDataProcessor() {
		return mDataProcessor;
	}

	/**
	 * @param dataProcessor the dataProcessor to set
	 */
	public void setDataProcessor(DataProcessor dataProcessor) {
		mDataProcessor = dataProcessor;
	}

	/**
	 * @return the transactionManager
	 */
	public PlatformTransactionManager getTransactionManager() {
		return mTransactionManager;
	}

	/**
	 * @param transactionManager the transactionManager to set
	 */
	public void setTransactionManager(PlatformTransactionManager transactionManager) {
		mTransactionManager = transactionManager;
	}

	public ApplicationVersion getApplicationVersion(String appCode, String versionName)
	{
		appCode = appCode == null ? "Undefined" : appCode;
		versionName = versionName == null ? "Undefined" : versionName;
		
		Element elt = mVersionCache.get(new AppVersionKey(appCode, versionName));
		return (ApplicationVersion) elt.getObjectValue();
	}
	
	public Environment getEnvironment(String orgCode, String envCode)
	{
		orgCode = orgCode == null ? "Undefined" : orgCode;
		envCode = envCode == null ? "Undefined" : envCode;
		
		Element elt = mEnvironmentCache.get(new OrgEnvKey(orgCode, envCode));
		return (Environment) elt.getObjectValue();
	}
	
	public Host getHost(String hostName, Environment env)
	{
		hostName = hostName == null ? "Undefined" : hostName;
		
		Element elt = mHostCache.get(new EnvHostKey(env, hostName));
		return (Host) elt.getObjectValue();
		
	}
	
	public Webapp getWebapp(String webappName, Application app)
	{
		webappName = webappName == null ? "Undefined" : webappName;
		
		Element elt = mWebappCache.get(new WebappKey(app, webappName));
		return (Webapp) elt.getObjectValue();
	}

	/**
	 * @return the applicationDao
	 */
	public ApplicationDao getApplicationDao() {
		return mApplicationDao;
	}

	/**
	 * @param applicationDao the applicationDao to set
	 */
	public void setApplicationDao(ApplicationDao applicationDao) {
		mApplicationDao = applicationDao;
	}

	/**
	 * @return the environmentDao
	 */
	public EnvironmentDao getEnvironmentDao() {
		return mEnvironmentDao;
	}

	/**
	 * @param environmentDao the environmentDao to set
	 */
	public void setEnvironmentDao(EnvironmentDao environmentDao) {
		mEnvironmentDao = environmentDao;
	}

	/**
	 * @return the perfDao
	 */
	public PerfDao getPerfDao() {
		return mPerfDao;
	}

	/**
	 * @param perfDao the perfDao to set
	 */
	public void setPerfDao(PerfDao perfDao) {
		mPerfDao = perfDao;
	}

	/* (non-Javadoc)
	 * @see org.appspy.server.service.impl.DataCollectionUtils#getClientHost(java.lang.String)
	 */
	public ClientHost getClientHost(String clientHostIP) {
		
		clientHostIP = clientHostIP == null ? "Unknown" : clientHostIP;
		
		Element elt = mClientHostCache.get(clientHostIP);
		return (ClientHost) elt.getObjectValue();
	}

	/* (non-Javadoc)
	 * @see org.appspy.server.service.impl.DataCollectionUtils#getSession(java.lang.String, org.appspy.server.bo.ApplicationVersion, org.appspy.server.bo.Environment, org.appspy.server.bo.User)
	 */
	public Session getSession(String sessionCode, ApplicationVersion version,
			Environment env, User user, ClientHost clientHost) {
		
		Session result = null;
		
		if (sessionCode != null)
		{
			Element elt = mSessionCache.get(new SessionKey(sessionCode, version, env, user, clientHost));
			result = (Session) elt.getObjectValue();
		}
		
		return result;
	}

	/* (non-Javadoc)
	 * @see org.appspy.server.service.impl.DataCollectionUtils#getUser(java.lang.String)
	 */
	public User getUser(String userName) {
		
		userName = userName == null ? "Anonymous" : userName;
		
		Element elt = mUserCache.get(userName);
		
		return (User) elt.getObjectValue();
	}

	/* (non-Javadoc)
	 * @see org.appspy.server.service.impl.DataCollectionUtils#getURL(java.lang.String, org.appspy.server.bo.Webapp)
	 */
	public URL getURL(String urlName, Webapp webapp) {
		
		urlName = urlName == null ? "Undefined" : urlName;
		
		Element elt = mURLCache.get(new URLKey(webapp, urlName));
		
		return (URL) elt.getObjectValue();
	}
	
	/* (non-Javadoc)
     * @see org.appspy.server.service.impl.DataCollectionUtils#getDelayType(int)
     */
    public DelayType getDelayType(int type)
    {
	    Element elt = mDelayTypeCache.get(type);
	    
	    return (DelayType) elt.getObjectValue();
    }

	/* (non-Javadoc)
     * @see org.appspy.server.service.impl.DataCollectionUtils#getHitResult(int)
     */
    public HitResult getHitResult(int res)
    {
	    Element elt = mHitResultCache.get(res);
	    
	    return (HitResult) elt.getObjectValue();
    }

	/**
     * @return the cacheManager
     */
    public CacheManager getCacheManager()
    {
    	return mCacheManager;
    }

	/**
     * @param cacheManager the cacheManager to set
     */
    public void setCacheManager(CacheManager cacheManager)
    {
    	mCacheManager = cacheManager;
    }

	/* (non-Javadoc)
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    public void afterPropertiesSet() throws Exception
    {
	    Cache versionCache = mCacheManager.getCache("org.appspy.versionCache");
	    mVersionCache = new SelfPopulatingCache(versionCache, new CacheEntryFactory(){

			public Object createEntry(Object key) throws Exception
            {
				AppVersionKey k = (AppVersionKey) key;
				ApplicationVersion version = mApplicationDao.findOrCreateVersion(k.getApp(), k.getVersion());
				
				return version;
            }});
	    
	    Cache environmentCache = mCacheManager.getCache("org.appspy.environmentCache");
	    mEnvironmentCache = new SelfPopulatingCache(environmentCache, new CacheEntryFactory(){

			public Object createEntry(Object key) throws Exception
            {
	            OrgEnvKey k = (OrgEnvKey) key;
	    		Environment env = mEnvironmentDao.findOrCreateEnvironment(k.getOrg(), k.getEnv());
	    		
	    		return env;
            }});
	    
	    Cache hostCache = mCacheManager.getCache("org.appspy.envHostCache");
	    mHostCache = new SelfPopulatingCache(hostCache, new CacheEntryFactory(){

			public Object createEntry(Object key) throws Exception
            {
				EnvHostKey k = (EnvHostKey) key;
				
				Host host = mEnvironmentDao.findOrCreateHost(k.getHost());
				
				if (!host.getEnvironments().contains(k.getEnv()))
				{
					host.getEnvironments().add(k.getEnv());
					mEnvironmentDao.updateHost(host);
				}
				
				return host;
            }});
	    
	    Cache webappCache = mCacheManager.getCache("org.appspy.webappCache");
	    mWebappCache = new SelfPopulatingCache(webappCache, new CacheEntryFactory(){

			public Object createEntry(Object key) throws Exception
            {
				WebappKey k = (WebappKey) key;
				Webapp webapp = mPerfDao.findOrCreateWebapp(k.getWebappName(), k.getApp());
				
				return webapp;
            }});
	    
	    Cache clientHostCache = mCacheManager.getCache("org.appspy.clientHostCache");
	    mClientHostCache = new SelfPopulatingCache(clientHostCache, new CacheEntryFactory(){

			public Object createEntry(Object key) throws Exception
            {
				String k = (String) key;
				
				return mPerfDao.findOrCreateClientHost(k);
            }});
	    
	    Cache userCache = mCacheManager.getCache("org.appspy.userCache");
	    mUserCache = new SelfPopulatingCache(userCache, new CacheEntryFactory(){

			public Object createEntry(Object key) throws Exception
            {
				String k = (String) key;
				
				return mPerfDao.findOrCreateUser(k);
            }});
	    
	    Cache sessionCache = mCacheManager.getCache("org.appspy.sessionCache");
	    mSessionCache = new SelfPopulatingCache(sessionCache, new CacheEntryFactory(){

			public Object createEntry(Object key) throws Exception
            {
				SessionKey k = (SessionKey) key;
				
				return mPerfDao.findOrCreateSession(k.getSessionCode(), k.getVersion(), k.getEnvironment(), k.getUser(), k.getClientHost());
            }});
	    
	    Cache urlCache = mCacheManager.getCache("org.appspy.URLCache");
	    mURLCache = new SelfPopulatingCache(urlCache, new CacheEntryFactory(){

			public Object createEntry(Object key) throws Exception
            {
	            URLKey k = (URLKey) key;
	            
	            return mPerfDao.findOrCreateURL(k.getURL(), k.getWebapp());
            }});
	    
	    Cache hitResultCache = mCacheManager.getCache("org.appspy.hitResultCache");
	    mHitResultCache = new SelfPopulatingCache(hitResultCache, new CacheEntryFactory(){

			public Object createEntry(Object key) throws Exception
            {
				int k = (Integer) key;
				
	            return mPerfDao.findOrCreateHitResult(k);
            }});
	    
	    Cache delaytypeCache = mCacheManager.getCache("org.appspy.delayTypeCache");
	    mDelayTypeCache = new SelfPopulatingCache(delaytypeCache, new CacheEntryFactory(){

			public Object createEntry(Object key) throws Exception
            {
	            int k = (Integer) key;
	            
	            return mPerfDao.findOrCreateDelayType(k);
            }});
    }
}
