/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.server.service.impl;

import java.util.ArrayList;
import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.appspy.core.data.CollectedData;
import org.appspy.server.bo.Hit;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class DataProcessorImpl implements DataProcessor {

	protected static final Log sLog = LogFactory.getLog(DataProcessorImpl.class);

	protected Collection<DataHandler> mDataHandlers = new ArrayList<DataHandler>();

	/* (non-Javadoc)
	 * @see org.appspy.server.service.impl.DataProcessor#process(org.appspy.core.data.CollectedData, org.appspy.server.bo.Hit)
	 */
	public void process(CollectedData collectedData, Hit parent) {
		for (DataHandler dataHandler : mDataHandlers) {
			if (dataHandler.acceptData(collectedData))
			{
				sLog.info("DataHandler found for " + collectedData + " : " + dataHandler);
				Hit h = dataHandler.handleData(collectedData, parent);
				if (collectedData.getChildren() != null)
				{
					for (CollectedData child : collectedData.getChildren())
					{
						process(child, h);
					}
				}
			}
			break;
		}
	}

	/* (non-Javadoc)
	 * @see org.appspy.server.service.impl.DataProcessor#registerDataHandler(org.appspy.server.service.impl.DataHandler)
	 */
	public void registerDataHandler(DataHandler handler) {
		if (!mDataHandlers.contains(handler))
		{
			mDataHandlers.add(handler);
		}
	}
}
