/**
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 *
 * You may obtain a copy of the License at 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */
package org.appspy.server.service.impl.cache;

import java.io.Serializable;

import org.appspy.server.bo.ApplicationVersion;
import org.appspy.server.bo.ClientHost;
import org.appspy.server.bo.Environment;
import org.appspy.server.bo.User;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class SessionKey implements Serializable
{

	/**
     * 
     */
    private static final long serialVersionUID = 1L;
    
    protected String mSessionCode = null;
    protected ApplicationVersion mVersion = null;
    protected Environment mEnvironment = null;
    protected User mUser = null;
    protected ClientHost mClientHost = null;
    
    public SessionKey(String sessionCode, ApplicationVersion version,
			Environment env, User user, ClientHost clientHost)
    {
    	mSessionCode = sessionCode;
    	mVersion = version;
    	mEnvironment = env;
    	mUser = user;
    	mClientHost = clientHost;
    }

	/* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj)
    {
    	if (obj instanceof SessionKey)
    	{
    		SessionKey comp = (SessionKey) obj;
    		return (
    				comp.getClientHost().getId().equals(getClientHost().getId()) && 
    				comp.getEnvironment().getId().equals(getEnvironment().getId()) && 
    				comp.getSessionCode().equals(getSessionCode()) && 
    				comp.getUser().getId().equals(getUser().getId()) && 
    				comp.getVersion().getId().equals(getVersion().getId()));
    	}
	    return false;
    }

	/* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode()
    {
	    return (getClientHost().getId() + 
	    		getSessionCode() + 
	    		getEnvironment().getId() + 
	    		getUser().getId() + 
	    		getVersion().getId()).hashCode();
    }

	/**
     * @return the sessionCode
     */
    public String getSessionCode()
    {
    	return mSessionCode;
    }

	/**
     * @param sessionCode the sessionCode to set
     */
    public void setSessionCode(String sessionCode)
    {
    	mSessionCode = sessionCode;
    }

	/**
     * @return the version
     */
    public ApplicationVersion getVersion()
    {
    	return mVersion;
    }

	/**
     * @param version the version to set
     */
    public void setVersion(ApplicationVersion version)
    {
    	mVersion = version;
    }

	/**
     * @return the environment
     */
    public Environment getEnvironment()
    {
    	return mEnvironment;
    }

	/**
     * @param environment the environment to set
     */
    public void setEnvironment(Environment environment)
    {
    	mEnvironment = environment;
    }

	/**
     * @return the user
     */
    public User getUser()
    {
    	return mUser;
    }

	/**
     * @param user the user to set
     */
    public void setUser(User user)
    {
    	mUser = user;
    }

	/**
     * @return the clientHost
     */
    public ClientHost getClientHost()
    {
    	return mClientHost;
    }

	/**
     * @param clientHost the clientHost to set
     */
    public void setClientHost(ClientHost clientHost)
    {
    	mClientHost = clientHost;
    }

}
